import 'dart:async';
import 'dart:io';

import 'package:file_picker/file_picker.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:firebase_storage_hooks/firebase_storage_hooks.dart';
import 'package:firebase_storage_hooks/src/models/commit_handler.dart';
import 'package:firebase_storage_hooks/src/models/progress_information.dart';
import 'package:firebase_storage_hooks/src/models/storage_file_handler.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_hooks/flutter_hooks.dart';

/// Returns a [StorageFileResult] that can be used to first select a file and
/// then upload that file to Firebase Storage.
///
/// This delegates work to [FilePicker] and can be managed by calling methods
/// on [StorageFileResult.fileHandler].
///
/// ```dart
/// StorageFileResult.fileHandler.selectFile()
/// ```
///
/// Once a file has been selected, to commit the file to Firebase Storage, you
/// simply need to `commit` that file which will upload the file to Firebase.
///
/// When true, [generateUniqueFileName] will append a 5 character to the file
/// name to ensure it is unique in the database. This may not be desirable
/// however if you'd like the previous file to be deleted or overwritten.
///
/// Example:
///
/// 'my coolPicture.jpg' -> '6a9h4_my_coolPicture.jpg'
///
StorageFileHandler<PlatformFile, String> useStorageFile({
  required Reference reference,
  required FileType fileType,
  required String Function(String fileName) fileNameBuilder,
  List<String>? allowedExtensions,
  SettableMetadata? settableMetadata,
}) {
  final progressInformation = useState<ProgressInformation?>(null);
  final error = useState<Object?>(null);
  final stackTrace = useState<StackTrace?>(null);
  // `useRef` does not trigger a rebuild when the value changes.
  final streamSubscription = useRef<StreamSubscription?>(null);
  final isMounted = useIsMounted();
  final fileHandler = useFile(
    fileType: fileType,
    allowedExtensions: allowedExtensions,
  );

  // The path of the file in FirebaseStorage.
  final remoteFilePath = useState<String?>(null);
  final _error = error.value;
  final _stackTrace = stackTrace.value;

  void clearError() {
    error.value = null;
    stackTrace.value = null;
  }

  final _remoteFilePath = remoteFilePath.value;
  final AsyncSnapshotWithProgress<String, ProgressInformation> snapshot;

  final _progressInformation = progressInformation.value;

  if (_error != null && _stackTrace != null) {
    snapshot = AsyncSnapshotWithProgress<String, ProgressInformation>.withError(
      _remoteFilePath != null ? ConnectionState.done : ConnectionState.active,
      _error,
      _stackTrace,
    );
  } else if (_remoteFilePath != null) {
    snapshot = AsyncSnapshotWithProgress<String, ProgressInformation>.withData(
      ConnectionState.done,
      _remoteFilePath,
    );
  } else if (_progressInformation != null) {
    snapshot = AsyncSnapshotWithProgress<String, ProgressInformation>.waiting(
      _progressInformation,
    );
  } else {
    assert(_error == null && _stackTrace == null);
    snapshot =
        const AsyncSnapshotWithProgress<String, ProgressInformation>.nothing();
  }

  return StorageFileHandler(
    commit: ([PlatformFile? platformFile]) async {
      final completer = Completer<CommitHandler<String>>();
      if (streamSubscription.value != null) {
        completer.completeError('Stream upload already in progress');
      }

      clearError();

      final _platformFile = platformFile ?? fileHandler.snapshot.data;

      if (_platformFile != null) {
        progressInformation.value = const ProgressInformation(
          totalBytes: 1,
          bytesTransferred: 0,
        );

        try {
          final filePathName = _platformFile.name;
          final fileName = fileNameBuilder(filePathName);
          final bucketName = reference.bucket;
          final childReference = reference.child(fileName);
          final fileLocation = 'gs://$bucketName/${childReference.fullPath}';
          final UploadTask uploadTask;

          // TODO: Check if file exists with current name. It won't

          // Web must work off bytes, no filePath
          if (kIsWeb) {
            uploadTask = childReference.putData(
              _platformFile.bytes!,
              settableMetadata,
            );
          } else {
            final filePath = _platformFile.path;

            uploadTask = childReference.putFile(
              File(filePath!),
              settableMetadata,
            );
          }

          streamSubscription.value = uploadTask.snapshotEvents.listen(
            (TaskSnapshot snapshot) {
              if (snapshot.totalBytes !=
                      progressInformation.value?.totalBytes ||
                  snapshot.bytesTransferred !=
                      progressInformation.value?.bytesTransferred) {
                progressInformation.value = ProgressInformation(
                  totalBytes: snapshot.totalBytes,
                  bytesTransferred: snapshot.bytesTransferred,
                );
              }
            },
            onError: (Object _error, StackTrace _stackTrace) {
              error.value = _error;
              stackTrace.value = _stackTrace;
            },
            onDone: () {
              remoteFilePath.value = fileLocation;
            },
            cancelOnError: true,
          );

          await uploadTask;
          completer.complete(
            CommitHandler(
              value: fileLocation,
              revert: () async {
                await childReference.delete();
              },
            ),
          );
        } catch (_error, _stackTrace) {
          completer.completeError(error, _stackTrace);
          error.value = _error;
          stackTrace.value = _stackTrace;
        } finally {
          streamSubscription.value?.cancel();
          streamSubscription.value = null;

          if (isMounted()) {
            progressInformation.value = null;
          }
        }
      } else {
        completer.completeError(
          StateError(
            'Cannot commit a file that has not been selected yet.',
          ),
        );
      }

      return completer.future;
    },
    fileHandler: fileHandler,
    snapshot: snapshot,
  );
}
